@@ -1,6 +1,8 @@
 #!/usr/bin/env python3
 import os
 import sys
+import json
+import shutil
 import requests
 import argparse
 import subprocess
@@ -8,7 +10,7 @@ from pathlib import Path
 from apacheconfig import make_loader
 from pprint import pprint
 
-__version__ = '0.5.5'
+__version__ = '0.5.7'
 
 
 def printerr(x):
@@ -27,7 +29,9 @@ class DO_WP_Maintain():
                  verbose=False,
                  debug=False,
                  hume=False,
-                 skip_plugins=None):
+                 skip_plugins=None,
+                 skip_themes=None,
+                 path_to_wpcli=None):
 
         # Even higher priority
         self.hume = hume
@@ -50,6 +54,17 @@ class DO_WP_Maintain():
         self.verbose = verbose
         self.debug = debug
         # Other runtime checks:
+        if path_to_wpcli is None:  # Path not provided, search in path
+            self.path_to_wpcli = shutil.which('wp')
+        else:  # path provided, set it
+            self.path_to_wpcli = path_to_wpcli
+
+        # Now that we have a path_to_wpcli, test it:
+        if not self.test_wpcli_works():
+            msg = 'No executable for wp-cli or provided one is invalid.'
+            printerr(msg)
+            sys.exit(1)
+
         if self.hume:  # Test
             try:
                 import hume
@@ -78,6 +93,21 @@ https://github.com/buanzo/hume/wiki''')
                                'task': 'WPUPDATER'})
                 raise(RuntimeError(msg))
 
+        # Skip_Themes check needs to go after populating self.wp_list
+        self.skip_themes = []
+        if skip_themes is not None:
+            for item in skip_themes:
+                # Follows same logic as skip_plugins, see below
+                if self.valid_skip_theme_spec(item):
+                    self.skip_themes.append(item)
+                else:
+                    msg = '"{}" is not a valid theme name. Skipping.'.format(item)
+                    printerr(msg)
+                    if self.hume:
+                        self.Hume({'level': 'warning',
+                                   'msg': msg,
+                                   'task': 'WPUPDATER'})
+
         # Skip_Plugins check needs to go after populating self.wp_list
         self.skip_plugins = []
         if skip_plugins is not None:
@@ -166,8 +196,22 @@ https://github.com/buanzo/hume/wiki''')
         retObj['stderr'] = result.stderr.decode('utf-8')
         return(retObj)
 
+    def test_wpcli_works(self):
+        r = False  # Return False by default
+        v = ''
+        cmd = [self.path_to_wpcli, 'cli', 'version']
+        r = self._run(cmd)
+        try:  # Quick check. Get version as valid test.
+            o = r['stdout']
+            v = o.strip().split('WP-CLI ')[1]
+        except Exception:
+            pass
+        if v.count('.') > 0:
+            r = True  # only case r will be True
+        return(r)
+
     def wp_run(self, path, args):
-        cmd = ['/usr/bin/wp', '--no-color']
+        cmd = [self.path_to_wpcli, '--no-color']
         if self.allow_root is True:  # __init__ checks EUID and --allow-root
             cmd.append('--allow-root')
         cmd.append('--path={}'.format(path))
@@ -232,7 +276,19 @@ https://github.com/buanzo/hume/wiki''')
             for pluginName in wpl:
                 self.update_plugin(pluginName,path=path)
 
-    def skip_plugin_update(self,pluginName,path):
+    def skip_theme_update(self, themeName, path):
+        for item in self.skip_themes:
+            if item.count(':') == 0:
+                if themeName == item:
+                    return(True)
+            elif item.count(':') == 1:
+                # path:themeName
+                c = '{}:{}'.format(path, themeName)
+                if c == item:
+                    return(True)
+        return(False)
+
+    def skip_plugin_update(self, pluginName, path):
         for item in self.skip_plugins:
             if item.count(':') == 0:
                 # assume it is a pluginName global skip spec
@@ -248,13 +304,11 @@ https://github.com/buanzo/hume/wiki''')
     def update_plugin(self,pluginName,path):
         if self.skip_plugin_update(pluginName,path):
             if self.verbose:
-                printerr('Skipping update of "{}" in "{}"'.format(pluginName,
-                                                                  path))
+                printerr('Skipping update of plugin "{}" in "{}"'.format(pluginName, path))
             return
         args = ['plugin', 'update', pluginName]
         if self.verbose:
-            printerr('Updating Wordpress Plugin {} in {}'.format(pluginName,
-                                                                 path))
+            printerr('Updating Wordpress plugin {} in {}'.format(pluginName, path))
         r = self.wp_run(path=path, args=args)
         if r['status'] > 0:
             msg = 'Error updating plugin {} in {}: {}'.format(pluginName,
@@ -266,20 +320,23 @@ https://github.com/buanzo/hume/wiki''')
                            'msg': msg,
                            'task': 'WPUPDATER'})
 
-    def update_themes(self):
-        args = ['theme', 'update', '--all']
-        for site in self.wp_list:
-            path = site['path']
+    def update_themes(self, themeName, path):
+        if self.skip_theme_update(themeName,path):
             if self.verbose:
-                printerr('Updating All Wordpress Themes in {}'.format(path))
-            r = self.wp_run(path=path, args=args)
-            if r['status'] > 0:
-                msg = 'Error updating themes {}: {}'.format(path, r['stderr'])
-                printerr(msg)
-                if self.hume:
-                    self.Hume({'level': 'warning',
-                               'msg': msg,
-                               'task': 'WPUPDATER'})
+                printerr('Skipping update of theme "{}" in "{}"'.format(themeName, path))
+            return
+        args = ['theme', 'update', themeName]
+        if self.verbose:
+            printerr('Updating Wordpress theme {} in {}'.format(themeName, path))
+        r = self.wp_run(path=path, args=args)
+        if r['status'] > 0:
+            msg = 'Error updating theme {} in {}'.format(themeName, path, r['stderr']
+
+            printerr(msg)
+            if self.hume:
+                self.Hume({'level': 'warning',
+                           'msg': msg,
+                           'task': 'WPUPDATER'})
 
     def update_wpcli(self):
         args = ['cli', 'update', '--yes']
@@ -365,22 +422,30 @@ https://github.com/buanzo/hume/wiki''')
             return(False)
         return(False)
 
-    def _extract_documentroots(self, myDict, someList=None):
-        # Credits to https://www.reddit.com/user/pushme2/
-        # https://www.reddit.com/r/learnpython/comments/25im14/python_3_searching_recursively_through_a/
-        if someList is None:
-            myList = []
+    def valid_skip_theme_spec(self,item):
+        # Yes, this function is exactly like valid_skip_plugin_spec
+        # I might deduplicate the code, but I need to analyze
+        # And yes, this might be true for all of skip_theme/skip_plugin
+        # functionality.
+        # TODO: regex for wordpress plugin names
+        if item.count(':') == 0:  # yes yes i know i can optimize this
+            return(True)
+        elif item.count(':') == 1:
+            return(True)
         else:
-            myList = someList
-        for key, value in myDict.items():
-            if self.debug:
-                printerr('Processing key {} of type {}'.format(key,type(key)))
-            if isinstance(value, dict):
-                self._extract_documentroots(value, myList)
-            else:
-                if key == 'documentroot':
-                    myList.append(value)
-        return(myList)
+            return(False)
+        return(False)
+
+    def _extract_documentroots(self,config):
+        paths = []
+        flatsplit = json.dumps(config).split(',')
+        for item in flatsplit:
+            item = item.strip()
+            if item.count('documentroot') == 1:
+                item = item.split(': ')[1].replace('"','').replace("'",'')
+                paths.append(item)
+                continue
+        return(paths)
 
     def get_apache2_documentroots(self):
         documentroots = []
@@ -508,11 +573,22 @@ DocumentRoots from.''')
                         action='store_true',
                         dest='skip_wpcli_update',
                         help='Do not update WP-CLI on startup')
+    parser.add_argument('--path-to-wpcli',
+                        dest='path_to_wpcli',
+                        default=None,
+                        metavar='PATH',
+                        help='''Path to the wp binary. If not specified, it will be searched in PATH.''')
     parser.add_argument('--skip-plugin',
                         action='append',
                         dest='skip_plugins',
                         metavar='PLUGIN_NAME',
-                        help='''Skip updating the indicated plugin. Can be specified multiple names.
+                        help='''Skip updating the indicated plugin. Can be specified multiple times.
+Multiple values separated by commas are NOT allowed''')
+    parser.add_argument('--skip-theme',
+                        action='append',
+                        dest='skip_themes',
+                        metavar='THEME_NAME',
+                        help='''Skip updating the indicated theme. Can be specified multiple times.
 Multiple values separated by commas are NOT allowed''')
     parser.add_argument('--run',
                         action='append',
@@ -539,7 +615,8 @@ Example: --run="plugin install wp-fail2ban --activate"''')
                               verbose=args.verbose,
                               debug=args.debug,
                               hume=args.hume,
-                              skip_plugins=args.skip_plugins)
+                              skip_plugins=args.skip_plugins,
+                              path_to_wpcli=args.path_to_wpcli)
     except Exception as exc:
         printerr(exc)
         sys.exit(1)
